use strict;
use warnings;

package WebService::Shippo::Entities;
require WebService::Shippo::Object;
require WebService::Shippo::Resource;
require WebService::Shippo::Address;
require WebService::Shippo::CarrierAccount;
require WebService::Shippo::CustomsDeclaration;
require WebService::Shippo::CustomsItem;
require WebService::Shippo::Manifest;
require WebService::Shippo::Parcel;
require WebService::Shippo::Rate;
require WebService::Shippo::Refund;
require WebService::Shippo::Shipment;
require WebService::Shippo::ReturnShipment;
require WebService::Shippo::Transaction;

1;
